﻿using System;
using System.Collections.Generic;
using EnvelopeCreatorProject.DocusignAPI;
using EnvelopeCreatorProject.Models.Recipients.IdentityCheckBuilders;

namespace EnvelopeCreatorProject.Models.Recipients
{
    internal abstract class RecipientTypeBuilder
    {
        protected List<String> _recipients;


        internal RecipientTypeBuilder()
        {
            _recipients = new List<String>();
        }

        protected abstract string GetData(Recipient objRecipient);
        
        internal void AddRecipientData(Recipient objRecipient)
        {
            _recipients.Add(GetData(objRecipient));
        }

        protected string GetRecipientCommonInfo(Recipient objRecipient)
        {
            return
                string.Format(
                    "{0}recipientId{0}:{0}{1}{0},{0}accessCode{0}:{0}{2}{0},{3}{0}routingOrder{0}:{0}{4}{0}",
                    "\"",
                    objRecipient.ID,
                    objRecipient.AccessCode,
                    GetIdentityCheckBuilder(objRecipient.RecipientAuthentication.IDCheckConfigurationName).Build(objRecipient),
                    objRecipient.RoutingOrder
                    );
        }

        private IIdentityCheckBuilder GetIdentityCheckBuilder(string key)
        {
            var identityBuilders = new Dictionary<string, IIdentityCheckBuilder>() { 
                {"", new NoAuthTypeBuilder()},
                {"ID Check $", new IdCheckBuilderBuilder()},
                {"SMS Auth $", new SmsAuthTypeBuilder()},
                {"Phone Auth $", new PhoneAuthTypeBuilder()},
                {"Live ID", new SocialAuthTypeBuilder()},
                {"Facebook", new SocialAuthTypeBuilder()},
                {"Linkedin", new SocialAuthTypeBuilder()},
                {"Google", new SocialAuthTypeBuilder()},
                {"Salesforce", new SocialAuthTypeBuilder()},
                {"Twitter", new SocialAuthTypeBuilder()},
                {"Yahoo", new SocialAuthTypeBuilder()},
                {"OpenID", new SocialAuthTypeBuilder()}
            };

            if (string.IsNullOrEmpty(key)) return new NoAuthTypeBuilder();
            return (identityBuilders.ContainsKey(key)) ? identityBuilders[key] : new NoAuthTypeBuilder();
        }

        internal virtual string GetRecipientsData()
        {
            return string.Join(",", _recipients);
        }
    }
}
